<?php
/**
 * Shell - class to work with $wpdb global object
 */
class dbDps {
    /**
     * Execute query and return results
     * @param string $query query to be executed
     * @param string $get what must be returned - one value (one), one row (row), one col (col) or all results (all - by default)
     * @param const $outputType type of returned data
     * @return mixed data from DB
     */
    static public $query = '';
    static public function get($query, $get = 'all', $outputType = ARRAY_A) {
    }
    /**
     * Execute one query
     * @return query results
     */
    static public function query($query) {
    }
    /**
     * Get last insert ID
     * @return int last ID
     */
    static public function insertID() {
        global $wpdb;
        return $wpdb->insert_id;
    }
    /**
     * Get number of rows returned by last query
     * @return int number of rows
     */
    static public function numRows() {
        global $wpdb;
        return $wpdb->num_rows;
    }
    /**
     * Replace prefixes in custom query. Suported next prefixes:
     * #__  Wordpress prefix
     * ^__  Store plugin tables prefix (@see DPS_DB_PREF if config.php)
     * @__  Compared of WP table prefix + Store plugin prefix (@example wp_s_)
     * @param string $query query to be executed
     */
    static public function prepareQuery($query) {
        global $wpdb;
        return str_replace(
                array('#__', '^__', '@__'),
                array($wpdb->prefix, DPS_DB_PREF, $wpdb->prefix. DPS_DB_PREF),
                $query);
    }
    static public function getError() {
        global $wpdb;
        return $wpdb->last_error;
    }
    static public function lastID() {
        global $wpdb;
        return $wpdb->insert_id;
    }
    static public function timeToDate($timestamp = 0) {
        if($timestamp) {
            if(!is_numeric($timestamp))
                $timestamp = dateToTimestampDps($timestamp);
            return date('Y-m-d', $timestamp);
        } else {
            return date('Y-m-d');
        }
    }
    static public function dateToTime($date) {
        if(empty($date)) return '';
        if(strpos($date, DPS_DATE_DL)) return dateToTimestampDps($date);
        $arr = explode('-', $date);
        return dateToTimestampDps($arr[2]. DPS_DATE_DL. $arr[1]. DPS_DATE_DL. $arr[0]);
    }
    static public function exist($table) {
      global $wpdb;
      switch ($table) {
         case 'dps_modules':
            $res = $wpdb->get_var("SHOW TABLES LIKE '{$wpdb->prefix}dps_modules'");
         break;
         case 'dps_modules_type':
            $res = $wpdb->get_var("SHOW TABLES LIKE '{$wpdb->prefix}dps_modules_type'");
         break;
         case 'dps_page_as_img':
            $res = $wpdb->get_var("SHOW TABLES LIKE '{$wpdb->prefix}dps_page_as_img'");
         break;
         case 'dps_page_sort_order':
            $res = $wpdb->get_var("SHOW TABLES LIKE '{$wpdb->prefix}dps_page_sort_order'");
         break;
         case 'dps_usage_stat':
            $res = $wpdb->get_var("SHOW TABLES LIKE '{$wpdb->prefix}dps_usage_stat'");
         break;
      }
      return !empty($res);
   }
    static public function prepareHtml($d) {
        if(is_array($d)) {
            foreach($d as $i => $el) {
                $d[ $i ] = self::prepareHtml( $el );
            }
        } else {
            $d = esc_html($d);
        }
        return $d;
    }
	static public function prepareHtmlIn($d) {
		if(is_array($d)) {
            foreach($d as $i => $el) {
                $d[ $i ] = self::prepareHtml( $el );
            }
        } else {
            $d = wp_filter_nohtml_kses($d);
        }
        return $d;
    }
	static public function escape($data) {
		global $wpdb;
		return $wpdb->_escape($data);
	}
	static public function getAutoIncrement($table) {
	}
	static public function setAutoIncrement($table, $autoIncrement) {
	}
}
