<?php
class bookControllerDps extends controllerDps {
	private $_displayHtml = '';
	private $_displayId = 0;
	public function preview() {
		add_action('init', array($this, 'doPreview'), 99);
	}
	public function doPreview() {
		$paramsDesc = $this->getModule()->getBookParamsDesc();
		$get = reqDps::get('get');
		$this->_displayId = (int) reqDps::getVar('id');
		$getBookParams = array('id' => $this->_displayId);
		foreach($paramsDesc as $k => $v) {
			if(isset($get[ $k ]))
				$getBookParams[ $k ] = esc_attr($get[ $k ]);
		}
		$this->_displayHtml = $this->getModule()->displayBook($getBookParams, array('get_all_pages' => false));
		$this->getView()->displayBookPreview( array('bookHtml' => $this->_displayHtml, 'id' => $this->_displayId) );
		exit();
	}
	public function addPagesFromImages() {
		$res = new responseDps();
		$post = reqDps::get('post');
		if($this->getModel()->addPagesFromImages($post)) {
			$batchRes = $this->getModel()->getBatchResults();
			$res->addMessage(sprintf(__('Added %d from %d pages', DPS_LANG_CODE), $batchRes['success'], $batchRes['total']));
			// Update book params after import pages - as we will reload page, and some settings can be lost
			if(isset($post['bookParams'])
				&& !empty($post['bookParams'])
				&& isset($post['bookParams']['dps'])
				&& !empty($post['bookParams']['dps'])
			) {
				$updateTaxData = $post['bookParams'];
				$tag_ID = (int) $updateTaxData['tag_ID'];
				$taxonomy = $updateTaxData['taxonomy'];
				if($tag_ID && $taxonomy == DPS_BOOK) {
					wp_update_term( $tag_ID, $taxonomy, $updateTaxData );
				}
			}
		} else
			$res->pushError ($this->getModel()->getErrors());

		$addFrom = '';
		if(isset($post['batch_pages']['attach_id'])) {
			$addFrom = 'wpimages';
		} elseif(isset($post['batch_pages']['img_url'])) {
			$addFrom = 'ftp';
		}
		$res->ajaxExec();
	}
	public function parseFolderForImages() {
		$res = new responseDps();
		if(($filesList = $this->getModel()->parseFolderForImages(reqDps::get('post')))) {
			$res->addMessage(__('Done', DPS_LANG_CODE));
			$res->addData('filesList', $filesList);
		} else
			$res->pushError ($this->getModel()->getErrors());
		$res->ajaxExec();
	}
	public function exportToHtml() {
		add_action('init', array($this, 'doExportHtml'), 99);
	}
	public function doExportHtml() {
		$id = (int) reqDps::getVar('id');
		if($id) {
			echo $this->getView()->getExportHtml($id);
		}
		exit();
	}
	public function exportToPdf() {
		add_action('init', array($this, 'doExportPdf'), 99);
	}
	public function doExportPdf() {
		$id = (int) reqDps::getVar('id');
		if($id) {
			if(!$this->getModule()->canUsePdfExport()) {
				echo 'You need to have at least php 7.1 version to use this feature';
				exit();
			}
			require_once($this->getModule()->getModDir(). 'classes'. DS. 'dom2pdfnew'. DS. 'autoload.inc.php');

			$book = $this->getModel()->getById($id);
			$content = $this->getView()->getExportHtml($id, array('simple' => true));
			$width = 400;
			$height = 600;

			preg_match_all('/src="([^"]*)"/i', $content, $imgSrcRes);

			if($imgSrcRes && !empty($imgSrcRes[0]) && !empty($imgSrcRes[1])) {
				foreach($imgSrcRes[1] as $i => $url) {
					$imgSize = @getimagesize($url);

					if(isset($imgSize[0]) && isset($imgSize[1])) {
						$newHeight = $imgSize[1] * $width / $imgSize[0];
						$height = $newHeight > $height ? (int) $newHeight : $height;
					}
					$content = str_replace($imgSrcRes[0][ $i ], 'style="max-width:100% !important; height: auto;" src="'. DPS_SITE_URL.$url .'"', $content);
				}
			}

			$options = new Dompdf\Options();
			$options->set('is_remote_enabled', true);
			$options->set('enable_html5_parser', true);
			$options->set('defaultMediaType', 'all');
			$options->set('isFontSubsettingEnabled', true);
			$options->set('defaultFont', 'Helvetica');

			$dompdf = new Dompdf\Dompdf($options);
			$dompdf->loadHtml($content);
      $dompdf->setPaper('A4', 'potrait');
			$dompdf->render();

			header('Content-Description: File Transfer');
			header('Content-Type: application/octet-stream');
			header('Content-Disposition: attachment; filename="'.$book->name. '.pdf"');
			print $dompdf->output();
		}
		exit();
	}
	/**
	 * @see controller::getPermissions();
	 */
	public function getPermissions() {
		return array(
			DPS_USERLEVELS => array(
				DPS_ADMIN => array('preview', 'doPreview', 'addPagesFromImages', 'parseFolderForImages')
			),
		);
	}
}
