function dpsBook(bookData) {
	this._bookData = bookData;
	this._html = null;
	this._zoomEnabled = false;
	this._zoomedLevel = 0;
	this._zoomDragBinded = false;
	this._fullscreenEnabled = false;
	this._bookMaximize = false;
	this._isDragging = false;
	this._zoomTargetEls = [];
	this._zoomPrevEvent = null;
	// To get page iterator by it's ID
	this._pageIdToIter = {};
	this._pageIdToIterWasSet = false;
	this._audioPagingObj = null;
	this._pagesCount = 0;
	this._promoLink = null;
	this._foundPages = [];
	this._currentFoundOpenPage = 0;
	this._searchValues = {};
	this._anim = 300;
	this._orStageSize = {width: 0, height: 0};	// or == Original
	this._orBookSize = {width: 0, height: 0};
	this._orShellSize = {width: 0, height: 0};
	this._orToolbarSize = {width: 0, height: 0};
	this._stageSizeChanged = false;
	this._fakePages = [];
	this._zomScrollK = 3;
	this._baseHtml = '';
	this._zoomOutInitPos = {top: 0, left: 0};
	this._promoLinkInitPos = {top: 0, left: 0};
	this._maxZoomScroll = {top: 0, left: 0};
	this._toolbarInResponsive = false;
	this._init();
}
dpsBook.prototype._init = function(reinit) {
	var self = this
	,	stage = this.getStage();

	this._pagesCount = this._bookData.pages.length;
	if(parseInt(this._bookData.params['en_audio_paging'])
		&& this._bookData.params['audio_paging']
		&& this._bookData.params['audio_paging'] != ''
		&& typeof(Audio) !== 'undefined'
	) {
		this._audioPagingObj = new Audio( this._bookData.params['audio_paging'] );
	}
	if(!this._bookData.pages) {
		// !!!DO SOMETHING HERE!!!
		return false;
	}
	if(this._isHardcover()) {
		// First and second page - should be with hardcover
		this._bookData.pages[0].hardcover = true;
		this._bookData.pages[1].hardcover = true;
		// Last page - with hardcover
		this._bookData.pages[ this._bookData.pages.length-1 ].hardcover = true;
		// If number of pages - is even number - then pre-last page should be hardcover too
		if(this._bookData.pages.length % 2 === 0) {
			this._bookData.pages[ this._bookData.pages.length-2 ].hardcover = true;
		}
	}
	this._baseHtml = stage.parent('.dpsBookWrapper:first').html();
	this._bookData.params['en_gradients'] = parseInt(this._bookData.params['en_gradients']);
	this._bookData.params['en_page_shadow'] = parseInt(this._bookData.params['en_page_shadow']);
	this._html = jQuery('#book_'+ this._bookData.term_id).turn({
		gradients: this._bookData.params['en_gradients']
	,	acceleration: true
	,	autoCenter: true
	,	width: this._bookData.params['width']
	,	height: this._bookData.params['height']
	,	pages: this._pagesCount
	,	direction: this._bookData.params['transition_direction']
	,	duration: this._bookData.params['transition_duration']
	,	when: {
			turning: function(e, page, openedPageNums) {
				self._setDisplayPagesNum(openedPageNums);
				if(self._audioPagingObj) {
					// Stop audio in case it didn't finish prev. play
					if(self._audioPagingObj.currentTime) {
						self._audioPagingObj.pause();
						self._audioPagingObj.currentTime = 0;
					}
					// Play sound again
					self._audioPagingObj.play();
				}
				self._removeFakePages();
				self._removeBorderShadow();
			}
		,	turned: function(e, page, openedPageNums) {
				self._setDisplayPagesNum(openedPageNums);

				var displayPageNums = self._getDisplayPagesNum()
				,	toolbar = self.getToolbar();

				self.set('displayPageNums', displayPageNums);
				toolbar.find('.dpsTbPageNumTxt').val( displayPageNums.join(' - ')+ ' / '+ self._pagesCount );
				self._processHightlightSearch( page );
				self._checkBorderShadow();
				if(self._zoomEnabled) {
					self._adjustPosForZommed();
				} else {
					self._createFakePages();
				}
				if(self._fullscreenEnabled) {
					self._imgSetZoomSizes();
				}
			}
		,	missing: function(event, pages) {
				for(var i = 0; i < pages.length; i++) {
					self._addPage(pages[i]);
					// Should investigate here
					// It will pass here only 6 pages, near current required pages - and that is ok, no need to create "en_pages_dynamic_preload" option or something like that
					/*if(typeof(self._bookData.params.en_pages_dynamic_preload) != 'undefined'
						&& parseInt(self._bookData.params.en_pages_dynamic_preload) == 1
						&& pages[i] > parseInt(self._bookData.params.open_on_page) + 1
					) {
						// Load current and 2 other pages, and then - stop
						break;
					}*/
				}
			}
		}
	});
	if(!reinit) {
		jQuery(document).keydown(function(e){
			var arrayLeft = e.key == 'ArrowLeft' || e.keyCode == 37
			,	arrayRight = e.key == 'ArrowRight' || e.keyCode == 39
			,	spaceKey = e.key == ' ' || e.keyCode == 32;

			if(arrayLeft || arrayRight || spaceKey) {
				var isRtl = self.getParam('transition_direction') == 'rtl'
				,	moveToStart =  isRtl ? arrayRight : arrayLeft
				,	moveToEnd = isRtl ? (arrayLeft || spaceKey) : (arrayRight || spaceKey)
				,	curPages = self._getDisplayPagesNum()
				,	pageNum = 0;

				if(curPages.length) {
					if(curPages.length == 2) {		// middle of book
						if(moveToStart) {
							pageNum = curPages[0] > 1 ? (curPages[0] - 1) : 1;
						}
						if(moveToEnd) {
							pageNum = curPages[1] < self.getPagesCount() ? (curPages[1] + 1) : self.getPagesCount();
						}
					} else {
						if(moveToStart && curPages[0] == self.getPagesCount()) {	// last page
							pageNum = self.getPagesCount() - 1;
						}
						if(moveToEnd && curPages[0] == 1) {					// first page
							pageNum = 2;
						}
					}
					if(pageNum > 0) {
						self.getHtml().turn('page', pageNum);
					}

				}
			}
			if(e.key == 'Esc' || e.keyCode == 27) {
				if(self._zoomEnabled)
					self.zoomOut();
				if(self._fullscreenEnabled)
					self._fullscreenOff();
			}
		});
		this._openPageOnLoad();
		this._bookData.params['en_fake_pages'] = parseInt(this._bookData.params['en_fake_pages']);
	}
	this._createFakePages();
	this._checkBorderShadow();
};
dpsBook.prototype.getParam = function(key) {
	return this._bookData.params[ key ];
};
dpsBook.prototype.setParam = function(key, value) {
	this._bookData.params[ key ] = value;
	return this;
};
dpsBook.prototype.getPagesCount = function() {
	return this._pagesCount;
};
dpsBook.prototype._returnPromoLinkLocation = function() {
	this._prInit();
	if(this._promoLink) {
		this._promoLink.css({
			'position': 'absolute'
		,	'top': 0
		,	'right': 0
		});
	}
};
dpsBook.prototype._prInit = function() {
	this._promoLink = this.getStage().children('a[target="_blank"]');
	if(this._promoLink.length == 0)
		this._promoLink = false;
};
dpsBook.prototype.setPlLocation = function() {
	if(!this._zoomEnabled) return;
	this._prInit();
	if(this._promoLink) {
		if(this._fullscreenEnabled) {
			this._promoLink.css({
				'position': 'fixed'
			,	'right': 0
			,	'top': 0
			});
			return;
		}
		var	promoLinkPos = {top: 0, left: 0}
		,	docScrollTop = jQuery(document).scrollTop()
		,	docScrollLeft = jQuery(document).scrollLeft();
		if(!this._promoLinkInitPos.top) {
			var stage = this.getStage()
			,	stageOffset = stage.offset()
			,	promoWidth = this._promoLink.width() + 10;
			this._promoLinkInitPos.top = stageOffset.top;
			this._promoLinkInitPos.left = stageOffset.left + stage.width() - promoWidth - 35;
			this._promoLinkInitPos.width = promoWidth;
		}
		promoLinkPos.top = this._promoLinkInitPos.top - docScrollTop;
		promoLinkPos.left = this._promoLinkInitPos.left - docScrollLeft;
		this._promoLink.css({
			'position': 'fixed'
		,	'left': promoLinkPos.left
		,	'top': promoLinkPos.top
		,	'width': this._promoLinkInitPos.width+ 'px'
		});
	}
};
dpsBook.prototype._restorePromoLinkLocation = function() {
	if(this._promoLink) {
		this._promoLink.css({
			'top': 0
		,	'right': 0
		});
	}
};
dpsBook.prototype._isHardcover = function() {
	return (this._bookData.params['cover_type'] == 'hardcover');
};
dpsBook.prototype._openPageOnLoad = function() {
	var hashData = toeGetHashParams()
	,	openPageNumber = 0;	// Page number, that should be opened, from 1 to N
	if(hashData && hashData.length) {
		for(var i = 0; i < hashData.length; i++) {
			if(typeof(hashData[i]) !== 'string') continue;
			// It should be in format page123, where 123 - is page ID
			if(strpos(hashData[i], 'page') === 0) {
				var openPageId = parseInt(str_replace(hashData[i], 'page', ''));
				if(openPageId) {
					var pageIter = this._getPageIterById(openPageId);
					if(pageIter !== false) {
						openPageNumber = pageIter + 1;	// In turn we count from 1
						break;
					}
				}
			}
			// It should be in format open2, where 2 - is page iter
			if(strpos(hashData[i], 'open') === 0) {
				openPageNumber = parseInt(str_replace(hashData[i], 'open', ''));
				break;
			}
		}
	}
	if(!openPageNumber && parseInt(this._bookData.params['open_on_page'])) {
		openPageNumber = parseInt(this._bookData.params['open_on_page']);
	}
	if(parseInt(this._bookData.params['en_fullscreen_onclick'])
		&& openPageNumber != 1
		&& openPageNumber == this._pagesCount
		&& parseInt(openPageNumber % 2)
	) {
		openPageNumber = 0;	// set 0 if openPageNumber is not a first or a last odd page
	}
	if(openPageNumber && openPageNumber <= this._pagesCount) {
		this.getHtml().turn('page', openPageNumber);
	}
};
dpsBook.prototype._getPageIterById = function(pageId) {
	if(!this._pageIdToIterWasSet) {
		for(var i in this._bookData.pages) {
			this._pageIdToIter[ this._bookData.pages[i].ID ] = parseInt(i);
		}
		this._pageIdToIterWasSet = true;
	}
	return (typeof(this._pageIdToIter[pageId]) === 'undefined' ? false : this._pageIdToIter[pageId]);
};
dpsBook.prototype._addPage = function(page) {
	var html = jQuery('#book_'+ this._bookData.term_id),
		additionalClasses = this._bookData.pages[page - 1].has_indent ? 'dpsHasIndent' : '';
	// Create a new element for this page
	var element = jQuery('<div />', {});
	if(this._bookData.pages[page - 1].hardcover) {
		element.addClass('hard');
	}
	if(this._bookData.params['en_gradients']) {
		element.addClass('en_gradients');
	}
	// Add the page to the flipbook
	if (html.turn('addPage', element, page)) {
		// Add the initial HTML
		// It will contain a loader indicator
		element.html('<div class="loader"></div><div class="page-main-content ' + additionalClasses + '"></div>');
		// Load the page
		this._loadPage(page, element);
	}
	this._bookData.pages[page - 1].html_element = element;
};
dpsBook.prototype._loadPage = function(page, element) {
	if (this._bookData.pages[page-1].post_type == 'product') {
        element.find('.page-main-content').append(this._bookData.pages[page - 1].product);
	} else {
        element.find('.page-main-content').append(this._bookData.pages[page - 1].post_content);
    }
	var images = element.find('img')
	,	imgCount = images.length;

	var bookDiv = jQuery('#book_'+ this._bookData.term_id);
	var imgMaxHeight = bookDiv.find('.page-wrapper').height();
	images.css({'max-height':imgMaxHeight+'px'});
	images.css({'width':'auto'});

	// Hide loader after all images will be loaded
	if(imgCount) {
		images.bind('load error', function(){
			imgCount--;
			if(imgCount <= 0) {
				element.find('.loader').hide();
			}
		});
		setTimeout(function(){
			element.find('.loader').hide();
		}, 3000);
	} else {
		element.find('.loader').hide();
	}
	if(this._bookData.params['en_gradients']) {
		// If page - is an image - then we will place gradient element uppon it
		if(parseInt(this._bookData.pages[page-1].attach_id)) {
			var gradient = jQuery('<div class="gradients separateGradient" />');
			element.hasClass('even') ? gradient.addClass('even') : gradient.addClass('odd');
			gradient.addClass(this._bookData.params['transition_direction'])
			element.append(gradient);
			element.addClass('dpsAttachPage');
		} else {	// Else - not allowed to place gradient element to page as all it's html content will not be clickable
			element.addClass('gradients').addClass(this._bookData.params['transition_direction']);
		}
	}
};
dpsBook.prototype._setDisplayPagesNum = function(openedPageNums) {
	var displayPageNums = [];
	for(var i in openedPageNums) {
		if(parseInt(openedPageNums[i])) {
			displayPageNums.push( openedPageNums[i] );
		}
	}
	this.set('displayPageNums', displayPageNums);
};
dpsBook.prototype._getDisplayPagesNum = function() {
	return this.get('displayPageNums');
}
dpsBook.prototype.getHtml = function() {
	return this._html;
};
dpsBook.prototype.getData = function() {
	return this._bookData;
};
dpsBook.prototype.getStage = function() {
	return jQuery('#book_stage_'+ this._bookData.term_id);
};
dpsBook.prototype.getShell = function() {
	return jQuery('#book_shell_'+ this._bookData.term_id);
};
dpsBook.prototype.getToolbar = function() {
	return jQuery('#book_toolbar_'+ this._bookData.term_id);
};
dpsBook.prototype.getBookShell = function() {
	return jQuery('#book_shell_'+ this._bookData.term_id);
};
dpsBook.prototype.getFullscreenBtn = function() {
	return this.getToolbar().find('.dpsTbBtnFullscreen');
};
dpsBook.prototype.getSearchForm = function() {
	return this.getToolbar().find('.dpsTbSearchForm');
};
dpsBook.prototype.getSearchFormShell = function() {
	return this.getToolbar().find('.dpsTbSearchShell');
};
dpsBook.prototype.getPrevSearchBtn = function() {
	return this.getSearchFormShell().find('.dpsTbBtnPrevSearch');
};
dpsBook.prototype.getNextSearchBtn = function() {
	return this.getSearchFormShell().find('.dpsTbBtnNextSearch');
};
/**
 * Should be all, except zoom out btn
 */
dpsBook.prototype.getToolbarMain = function() {
	return this.getToolbar().find('.btnTbMain');
};
dpsBook.prototype.getZoomOutBtn = function() {
	return this.getStage().find('.dpsTbZoomOutShell');
};
dpsBook.prototype.set = function(key, val) {
	this._bookData[ key ] = val;
};
dpsBook.prototype.get = function(key) {
	return this._bookData[ key ];
};
dpsBook.prototype.isset = function(key) {
	typeof(this._bookData[ key ]) === 'undefined' ? false : true;
};
dpsBook.prototype.setZoomOutPos = function(stage) {
	if(!this._zoomEnabled) return;
	var	zoomBtn = this.getZoomOutBtn()
	,	incPos = 30;
	if(this._fullscreenEnabled) {
		zoomBtn.css({
			'left': incPos
		,	'bottom': zoomBtn.height() + incPos
		,	'position': 'fixed'
		});
		return;
	}
	var docScrollTop = jQuery(document).scrollTop()
	,	docScrollLeft = jQuery(document).scrollLeft()
	,	zoomOutPos = {top: 0, left: 0};
	if(!this._zoomOutInitPos.top) {
		var stage = stage ? stage : this.getStage()
		,	stageOffset = stage.offset()
		,	stageHeight = stage.height();
		this._zoomOutInitPos.top = stageOffset.top + stageHeight - incPos - zoomBtn.height();
		this._zoomOutInitPos.left = stageOffset.left + incPos;
	}
	zoomOutPos.top = this._zoomOutInitPos.top - docScrollTop;
	zoomOutPos.left = this._zoomOutInitPos.left - docScrollLeft;
	zoomBtn.css({
		'left': zoomOutPos.left
	,	'top': zoomOutPos.top
	,	'position': 'fixed'
	});
};
/**
 * Zoom book
 * @param {int} zoomLevel Zoom level
 */
dpsBook.prototype.zoomIn = function(zoomLevel) {
	var bookStage;

	this._zoomEnabled = true;

	zoomLevel = zoomLevel ? parseInt(zoomLevel) : this._bookData.params['zoom_level'];
	zoomLevel = !zoomLevel ? 2 : zoomLevel;	// Let check be here too - for some cases ...........
	this._zoomedLevel = zoomLevel;

	bookStage = this.getStage();
	this.set('prevStageOverflow', bookStage.css('overflow'));
	bookStage.css('overflow', 'scroll');
	this.getHtml().css('top', (this.getHtml().height() / 2) + 'px');
	this.getHtml().zoom( zoomLevel );
	this.getToolbarMain().hide();
	this.getZoomOutBtn().show();
	this._adjustPosForZommed();
	this._zoomMakeDraggable();
	this._imgSetZoomSizes();
	this._removeFakePages();
	this.toggleSearchForm(false);
	this.setZoomOutPos();
	this.setPlLocation();
};
dpsBook.prototype._imgSetZoomSizes = function(restore) {
	var bookHtml = this.getHtml()
	,	pageNums = this._getDisplayPagesNum()	// we need all display page numbers
	,	turnData = bookHtml.data()
	,	self = this
	,	savedImgs = {};
	for(var i in pageNums) {
		var pageNum = pageNums[i];
		if(turnData && turnData.pageObjs && turnData.pageObjs[ pageNum ]) {
			var page = turnData.pageObjs[ pageNum ]
			,	imgs = page.find('img');
			if(imgs && imgs.length) {
				imgs.each(function(){
					var attachId = toeGetImgAttachId(this)
					,	currentImg = this;
					if(restore)
						currentImg._dpsInZoom = false;
					if(attachId
						&& self._bookData.pages[ pageNum-1 ].attach_sizes
						&& self._bookData.pages[ pageNum-1 ].attach_sizes[ attachId ]
						&& self._bookData.pages[ pageNum-1 ].attach_sizes[ attachId ].full
						&& !self._bookData.pages[ pageNum-1 ].attach_sizes[ attachId ].not_transform
						&& !currentImg._dpsInZoom
					) {
						if(restore) {	// Restore small sizes
							if(self._bookData.pages[ pageNum-1 ].attach_sizes[ attachId ].on_page) {
								savedImgs[ attachId ] = currentImg;
								jQuery('<img />')
								.attr('src', self._bookData.pages[ pageNum-1 ].attach_sizes[ attachId ].on_page)
								.attr('data-attach-id', attachId)
								.attr('data-page-num', pageNum)
								.load(function(){
									var _attachId = jQuery(this).data('attach-id')
									,	_pageNum = jQuery(this).data('page-num');
									jQuery(savedImgs[ _attachId ]).attr('src', self._bookData.pages[ _pageNum-1 ].attach_sizes[ _attachId ].on_page);
									jQuery(this).remove();
								});
							}
						} else {	// Set full size for image
							var currentSrc = jQuery(this).attr('src');
							self._bookData.pages[ pageNum-1 ].attach_sizes[ attachId ].on_page = currentSrc;
							// Image already are in full size
							if(self._bookData.pages[ pageNum-1 ].attach_sizes[ attachId ].on_page == self._bookData.pages[ pageNum-1 ].attach_sizes[ attachId ].full) {
								self._bookData.pages[ pageNum-1 ].attach_sizes[ attachId ].not_transform = true;
							} else {
								// Create temp img - to not change src of current image until new src will be loaded
								savedImgs[ attachId ] = currentImg;
								currentImg._dpsInZoom= true;
								jQuery('<img />')
								.attr('src', self._bookData.pages[ pageNum-1 ].attach_sizes[ attachId ].full)
								.attr('data-attach-id', attachId)
								.attr('data-page-num', pageNum)
								.load(function(){
									var _attachId = jQuery(this).data('attach-id')
									,	_pageNum = jQuery(this).data('page-num');
									jQuery( savedImgs[ _attachId ] ).attr('src', self._bookData.pages[ _pageNum-1 ].attach_sizes[ _attachId ].full);
									jQuery(this).remove();
								});
							}
						}
					}
				});
			}
		}
	}
};
dpsBook.prototype._imgRestoreZoomSizes = function() {
	this._imgSetZoomSizes(true);
};
/**
 * Return book to normal zoom
 */
dpsBook.prototype.zoomOut = function() {
	this._zoomEnabled = false;

	var bookStage = this.getStage()
	,	bookStageParent = bookStage.parent('.dpsBookWrapper:first');

	bookStage.css('overflow', this.get('prevStageOverflow'));

	if(this._fullscreenEnabled) {
		this._fullscreenOff();
	}
	this.getToolbarMain().show();
	this.getZoomOutBtn().hide();
	this.center();
	this._zoomMakeUndraggable();
	this._imgRestoreZoomSizes();
	this.getHtml().turn('destroy');
	bookStageParent.hide();
	bookStageParent.html( this._baseHtml );
	this._init(true);
	bookStageParent.show();
	this.getHtml().turn('page', this.getHtml().turn('page'));
	this.getHtml().turn('center');
	this._restorePromoLinkLocation();
	this._createFakePages();
	this._returnPromoLinkLocation();
	this.adjustWidth( jQuery(window).width() );	// Adjust width after re-creation
	this._checkMaximizeBookRestore();

	if(this._fullscreenEnabled) {
		this._fullscreenOn();
		this.center();
	}
};
dpsBook.prototype.center = function() {
	var bookHtml = this.getHtml();
	bookHtml.turn('update');
	bookHtml.turn('center');
};
dpsBook.prototype._zoomBookHtmlMouseDownCb = function(e) {
	var book = e.data.book;
	e.preventDefault ? e.preventDefault() : e.returnValue = false;
	if(book)
		book._isDragging = true;
};
dpsBook.prototype._zoomDocMouseUpCb = function(e) {
	var book = e.data.book;
	if(book && book._isDragging) {
		book._isDragging = false;
		if(book._zoomTargetEls && book._zoomTargetEls.length) {
			for(var i in book._zoomTargetEls) {
				if(book._zoomTargetEls[i].element) {
					book._zoomTargetEls[i].element.isInDragList = false;
					jQuery(book._zoomTargetEls[i].element).css('cursor', book._zoomTargetEls[i].cursor);
				}
			}
			book._zoomTargetEls[i] = [];
		}
	}
};
dpsBook.prototype._zoomDocMouseMoveCb = function(e) {
	var book = e.data.book;
	book._isDragging = true;
	// Always drag when move mouse
	if(book && book._isDragging) {
		if(book._zoomPrevEvent) {
			var xDisplace = e.clientX - book._zoomPrevEvent.clientX
			,	yDisplace = e.clientY - book._zoomPrevEvent.clientY
			,	bookStage = book.getStage()
			,	stageScrollLeft = bookStage.scrollLeft()
			,	stageScrollTop = bookStage.scrollTop()
			,	newScrollLeft = stageScrollLeft + xDisplace * book._zomScrollK
			,	newScrollTop = stageScrollTop + yDisplace * book._zomScrollK;

			if(newScrollLeft < 0)
				newScrollLeft = 0;
			/*else if(book._maxZoomScroll.left > 0 && newScrollLeft > book._maxZoomScroll.left)
				newScrollLeft = book._maxZoomScroll.left;*/

			if(newScrollTop < 0)
				newScrollTop = 0;
			/*else if(book._maxZoomScroll.top > 0 && newScrollTop > book._maxZoomScroll.top)
				newScrollTop = book._maxZoomScroll.top;*/

			bookStage
				.scrollLeft( newScrollLeft )
				.scrollTop( newScrollTop );
		}
		book._zoomPrevEvent = e;

	}
};
dpsBook.prototype._zoomMakeDraggable = function() {
	jQuery(document)
		.bind('mousemove', {book: this}, this._zoomDocMouseMoveCb);
};
dpsBook.prototype._zoomMakeUndraggable = function() {
	jQuery(document)
		.unbind('mousemove', this._zoomDocMouseMoveCb);
};
dpsBook.prototype.fullscreenToggle = function() {
	if(this._fullscreenEnabled) {
		this._fullscreenOff();
		this._fullscreenEnabled = false;
	} else {
		this._fullscreenOn();
		this._fullscreenEnabled = true;
		if(this._bookMaximize)
			this._bookMaximize = false;
	}
	this.getHtml().turn('center');
	this._createFakePages();
};
dpsBook.prototype.fullscreenOnByClick = function() {
	if(!this._fullscreenEnabled && parseInt(this._bookData.params['en_fullscreen_onclick'])) {
		this._checkMaximizeBookRestore();
		this._fullscreenOn();
		this._fullscreenEnabled = true;
		if(this._bookMaximize)
			this._bookMaximize = false;
		this.getHtml().turn('center');
		this._createFakePages();
		return false;	// To prevent default action by mouse click on page
	}
};
dpsBook.prototype._fullscreenOn = function() {
	this.resizeBook();
	var fsBtn = this.getFullscreenBtn()
	,	html = this.getHtml()
	,	stage = this.getStage()
	,	bookShell = this.getBookShell()
	,	toolbar = this.getToolbarMain();

	this.set('fsPrevTop', (this.isset('fsPrevTop') ? this.get('fsPrevTop') : html.css('top')));
	this.set('fsPrevShellHeight', (this.isset('fsPrevShellHeight') ? this.get('fsPrevShellHeight') : bookShell.css('height')));
	this.set('stagePrevStyle', (this.isset('stagePrevStyle') ? this.get('stagePrevStyle') : stage.attr('style')));

	stage.css({
		'position': 'fixed'
	,	'min-width': '100%'
	,	'min-height': '100%'
	,	'left': '0'
	,	'top': '0'
	,	'z-index': '110000'
	});
	toolbar.css({
		'position': 'fixed'
	,	'min-width': '100%'
	,	'left': '0'
	,	'bottom': '0'
	});
	bookShell.css({
		'height': stage.height() - toolbar.height()
	});
	html.css({
		'top': Math.round((jQuery(window).height() - toolbar.height() - html.height()) / 2)
	});
	var toolbarCustomBtnsColor = '';
	if(this._bookData.params.toolbar_btns_color) {
		switch(this._bookData.params.toolbar_code) {
			case 'simple': default:
				toolbarCustomBtnsColor = 'color:'+ this._bookData.params.toolbar_btns_color + ';';
				break;
			/*case 'night':
				toolbarCustomBtnsColor = 'color:'+ this._bookData.params.toolbar_btns_color + ';';
					//'background: ' + this._bookData.params.toolbar_btns_color +
					//'; -webkit-background-clip: '+ 'text' +
					//'; -webkit-text-fill-color: '+ 'transparent'+ ';';
				break;*/
		}
	}
	fsBtn.addClass('dpsTbBtnFullscreenEnabled').html('<i class="fa fa-compress " style="'+ toolbarCustomBtnsColor +'"></i>');
	this._toolbarReinitResponsive();
};
dpsBook.prototype._fullscreenOff = function() {
	var fsBtn = this.getFullscreenBtn()
	,	html = this.getHtml()
	,	stage = this.getStage()
	,	bookShell = this.getBookShell()
	,	toolbar = this.getToolbarMain()
	,	stagePrevStyle = this.get('stagePrevStyle');
	if(!stagePrevStyle)
		stagePrevStyle = '';
	/*stage.css({
		'position': 'relative'
	,	'min-width': 'auto'
	,	'min-height': 'auto'
	});*/
	stage.attr('style', stagePrevStyle);
	toolbar.css({
		'position': 'static'
	,	'min-width': 'auto'
	});
	bookShell.css({
		'height':  this.get('fsPrevShellHeight')
	});
	html.css({
		'top': this.get('fsPrevTop')
	});
	var toolbarCustomBtnsColor = ''
	,	toolbarHtml = '';
	if(this._bookData.params.toolbar_btns_color) {
		toolbarCustomBtnsColor = 'color:'+ this._bookData.params.toolbar_btns_color + ';';
	}
	switch(this._bookData.params.toolbar_code) {
		case 'simple': default:
			toolbarHtml = '<i class="fa fa-arrows-alt " style="'+ toolbarCustomBtnsColor +'"></i>';
			break;
	}
	fsBtn.removeClass('dpsTbBtnFullscreenEnabled').html(toolbarHtml);
	this.center();
	this.resizeBook(true);
	this._toolbarReinitResponsive();
};
/**
 * Makes an increasing of book size for Fullscreen mode
 * @param {bool} restore Restore the original book size
 */
dpsBook.prototype.resizeBook = function(restore) {
	var bookHtml = this.getHtml()
	,	bookStage = this.getStage()
	,	bookStageWidth = jQuery(window).width()
	,	bookStageHeight = jQuery(window).height() - this.getToolbar().height()
	,	newBookSizes = [];
	if(restore) {
		if(this._bookData.params['base_width'] && this._bookData.params['base_height']) {
			this._bookData.params['width'] = this._bookData.params['base_width'];
			this._bookData.params['height'] = this._bookData.params['base_height'];
		}
	} else {
		var bookWidth = parseInt(this._bookData.params['width'])
		,	bookHeight = parseInt(this._bookData.params['height']);
		if(bookWidth > bookHeight) {
			newBookSizes = this.recalsBookWidth(bookStageWidth, bookWidth, bookHeight);
			if(newBookSizes['height'] > (bookStageHeight - 80)) {
				newBookSizes = this.recalcBookHeight(bookStageHeight, newBookSizes['width'], newBookSizes['height'], true);
			}
		} else {
			newBookSizes = this.recalcBookHeight(bookStageHeight, bookWidth, bookHeight);
			if(newBookSizes['width'] > (bookStageWidth - 80)) {
				newBookSizes = this.recalsBookWidth(bookStageWidth, newBookSizes['width'], newBookSizes['height'], true);
			}
		}
		this._bookData.params['base_width'] = this._bookData.params['width'];
		this._bookData.params['base_height'] = this._bookData.params['height'];
		this._bookData.params['width'] = newBookSizes['width'];
		this._bookData.params['height'] = newBookSizes['height'];
	}
	var currentPage = bookHtml.turn('page')
	,	bookStageParent = bookStage.parent();

	bookHtml.turn('destroy');
	bookStageParent.hide();
	bookStageParent.html( this._baseHtml );
	this._init(true);
	bookHtml = this.getHtml();
	bookStageParent.show();
	bookHtml.turn('page', currentPage);
	bookHtml.turn('center');
	this._restorePromoLinkLocation();
	this._createFakePages();
	this._returnPromoLinkLocation();
	if(restore) {
		this._imgRestoreZoomSizes();
		// Adjust width after re-creation
		this.adjustWidth( jQuery(window).width() );
		this.setBookTop(bookHtml);
	} else {
		this._imgSetZoomSizes();
	}
};
dpsBook.prototype.recalsBookWidth = function(shell, width, height, recalc) {
	var bookSizes = [];
	bookSizes['width'] = shell - 80;
	if(recalc) {
		var prop = bookSizes['width'] / width;
		bookSizes['width'] = Math.round(width * prop);
		bookSizes['height'] = Math.round(height * prop);
	} else {
		bookSizes['height'] = Math.round(bookSizes['width'] * height / width);
	}
	return bookSizes;
};
dpsBook.prototype.recalcBookHeight = function(shell, width, height, recalc) {
	var bookSizes = [];
	bookSizes['height'] = shell - 80;
	if(recalc) {
		var prop = bookSizes['height'] / height;
		bookSizes['width'] = Math.round(width * prop);
		bookSizes['height'] = Math.round(height * prop);
	} else {
		bookSizes['width'] = Math.round(width * bookSizes['height'] / height);
	}
	return bookSizes;
};
dpsBook.prototype._checkMaximizeBookRestore = function() {
	var restore = this._bookMaximize ? true : false;

	if(restore)
		this.maximizeBook(restore);
};
/**
 * Makes an increasing of book size at the first show, if "Fullscreen On Click" option is enabled
 * @param {bool} restore Restore the original book size
 */
dpsBook.prototype.maximizeBook = function(restore) {
	restore = restore ? restore : false;

	if(parseInt(this._bookData.params['en_fullscreen_onclick'])) {
		var bookHtml = this.getHtml()
		,	bookStage = this.getStage()
		,	bookStageWidth = bookStage.width()
		,	bookStageHeight = bookStage.height() - this.getToolbar().height()
		,	newBookSizes = [];
		if(restore) {
			if(this._bookData.params['cur_width'] && this._bookData.params['cur_height']) {
				this._bookData.params['width'] = this._bookData.params['cur_width'];
				this._bookData.params['height'] = this._bookData.params['cur_height'];
				this._bookMaximize = false;
			}
		} else {
			var bookWidth = parseInt(this._bookData.params['width'])
			,	bookHeight = parseInt(this._bookData.params['height']);
			newBookSizes['height'] = Math.round(bookStageHeight * 0.95);
			newBookSizes['width'] = Math.round(bookWidth * newBookSizes['height'] / bookHeight);
			if(newBookSizes['width'] > (bookStageWidth * 2)) {
				var curWidth = newBookSizes['width'];
				newBookSizes['width'] = Math.round(bookStageWidth * 2 * 0.95);
				newBookSizes['height'] = Math.round(newBookSizes['width'] * newBookSizes['height'] / curWidth);
			}
			this._bookData.params['cur_width'] = this._bookData.params['width'];
			this._bookData.params['cur_height'] = this._bookData.params['height'];
			this._bookData.params['width'] = newBookSizes['width'];
			this._bookData.params['height'] = newBookSizes['height'];
			this._bookMaximize = true;
		}
		var currentPage = bookHtml.turn('page')
		,	bookStageParent = bookStage.parent();

		bookHtml.turn('destroy');
		bookStageParent.hide();
		bookStageParent.html( this._baseHtml );
		this._init(true);
		bookHtml = this.getHtml();
		bookStageParent.show();
		bookHtml.turn('page', currentPage);
		bookHtml.turn('center');
		this._restorePromoLinkLocation();
		this._createFakePages();
		this._returnPromoLinkLocation();
		if(restore) {
			// Adjust width after re-creation
			this.adjustWidth( jQuery(window).width() );
		} else {
			// Adjust width after re-creation
			this.adjustWidth( jQuery(window).width(), true );
		}
		this.setBookTop(bookHtml);
	}
};
dpsBook.prototype.setBookTop = function(bookHtml) {
	var bookTop = Math.round((this.getStage().height() - this.getToolbar().height() - parseInt(bookHtml.height())) / 2);
	bookHtml.css('top', bookTop + 'px');
}
dpsBook.prototype.toggleSearchForm = function(newValue) {
	if(typeof(newValue) !== 'undefined') {
		newValue
			? this.getSearchFormShell().show( this._anim )
			: this.getSearchFormShell().hide( this._anim );
	} else
		this.getSearchFormShell().toggle( this._anim );
};
dpsBook.prototype.search = function(value) {
	var tmpDiv = document.createElement('div');
	this._foundPages = [];
	this._searchValues = {};

	if(!value || value == '') {
		this._processHightlightSearch();
		return;
	}
	var valueLowered = value.toLowerCase();
	for(var i in this._bookData.pages) {
		if (this._bookData.pages[ i ].post_type == 'product') {
            tmpDiv.innerHTML = this._bookData.pages[i].product;
        } else {
            tmpDiv.innerHTML = this._bookData.pages[i].post_content;
        }
		var pageText = tmpDiv.textContent || tmpDiv.innerText || ''
		,	pageTextLowered = pageText ? pageText.toLowerCase() : ''
		,	startFound = false;

		if(pageText && pageText != '' && (startFound = strpos(pageTextLowered, valueLowered)) !== false) {
			var pageI = parseInt(i);
			this._foundPages.push( pageI );
			if( !this._searchValues[ pageI ] )
				this._searchValues[ pageI ] = [];
			while(startFound !== false) {
				var lastFound = pageText.substr(startFound, valueLowered.length);
				if(toeInArray(this._searchValues[ pageI ], lastFound) === -1)
					this._searchValues[ pageI ].push( lastFound );
				pageTextLowered = pageTextLowered.substr(startFound + 1);
				pageText = pageText.substr(startFound + 1);
				if(pageTextLowered && pageTextLowered != '')
					startFound = strpos(pageTextLowered, valueLowered);
				else
					startFound = false;
			}
		}
	}
	if(this._foundPages.length) {
		/*if(this._foundPages.length > 1)
			nextSearchBtn.addClass('dpsTbBtnSearchActive');*/
		var book = this.getHtml();
		if(book.turn('page') == (this._foundPages[0] + 1))
			this._processHightlightSearch();
		book.turn('page', this._foundPages[0] + 1);
	} else
		this._processHightlightSearch();
};
dpsBook.prototype.goToPrevSearch = function() {
	if(this._foundPages.length > 1) {
		var book = this.getHtml()
		,	currentPage = parseInt(book.turn('page'));
		if(this._currentFoundOpenPage > 0) {
			this._currentFoundOpenPage--;
			var nextSearchPage = this._foundPages[ this._currentFoundOpenPage ] + 1;
			if(currentPage % 2 !== 0 && currentPage - 1 === nextSearchPage && this._currentFoundOpenPage > 0) {
				this._currentFoundOpenPage--;
				nextSearchPage = this._foundPages[ this._currentFoundOpenPage ] + 1;
			}
			book.turn('page', nextSearchPage);
			this._checkMaximizeBookRestore();
		}
		//this.__checkCanSearchMove();
	}
};
dpsBook.prototype.goToNextSearch = function() {
	if(this._foundPages.length > 1) {
		var book = this.getHtml()
		,	currentPage = parseInt(book.turn('page'));
		if(this._currentFoundOpenPage < (this._foundPages.length - 1)) {
			this._currentFoundOpenPage++;
			var nextSearchPage = this._foundPages[ this._currentFoundOpenPage ] + 1;
			if(currentPage % 2 === 0 && currentPage + 1 === nextSearchPage) {
				this._currentFoundOpenPage++;
				nextSearchPage = this._foundPages[ this._currentFoundOpenPage ] + 1;
			}
			book.turn('page', nextSearchPage);
			this._checkMaximizeBookRestore();
		}
		//this.__checkCanSearchMove();
	}
};
/*dpsBook.prototype.__checkCanSearchMove = function() {
	var canMoveNext = false
	,	canMovePrev = false
	,	nextSearchBtn = this.getNextSearchBtn()
	,	prevSearchBtn = this.getPrevSearchBtn();
	if(this._currentFoundOpenPage < (this._foundPages.length - 1))
		canMoveNext = true;
	if(this._currentFoundOpenPage > 0)
		canMovePrev = true;
	if(canMoveNext) {
		nextSearchBtn.addClass('dpsTbBtnSearchActive');
	} else {
		nextSearchBtn.removeClass('dpsTbBtnSearchActive');
	}
	if(canMovePrev) {
		prevSearchBtn.addClass('dpsTbBtnSearchActive');
	} else {
		prevSearchBtn.removeClass('dpsTbBtnSearchActive');
	}
};*/
dpsBook.prototype._processHightlightSearch = function() {
	var displayPagesNum = this._getDisplayPagesNum();
	if(displayPagesNum && displayPagesNum.length) {
		for(var i = 0; i < displayPagesNum.length; i++) {
			var pageI = displayPagesNum[i] - 1
			,	page = this._bookData.pages[ pageI ];
			this._removeHightlightFromElement(page.html_element);
			if(this._searchValues
				&& this._searchValues[ pageI ]
				&& this._searchValues[ pageI ].length
				&& page
				&& toeInArray(pageI, this._foundPages) !== -1
			) {
				var innerHTML = page.html_element.html();
				for(var sI in this._searchValues[ pageI ] ) {
					var replaceData = {};
					replaceData[ this._searchValues[ pageI ][sI] ] = '[DPS_HIGHLIGHT_START]'+ this._searchValues[ pageI ][sI]+ '[DPS_HIGHLIGHT_END]';
					innerHTML = replaceWords(innerHTML, replaceData);
					innerHTML = str_replace(innerHTML, '[DPS_HIGHLIGHT_START]', '<span class="dpsSearchHightlight">');
					innerHTML = str_replace(innerHTML, '[DPS_HIGHLIGHT_END]', '</span>');
				}
				page.html_element.html( innerHTML );
			}
		}
	}
};
dpsBook.prototype._removeHightlightFromElement = function(element) {
	element.find('.dpsSearchHightlight').each(function(){
		var outerHtml = this.outerHTML
		,	innerSearchText = jQuery(this.outerHTML).get(0).innerHTML;
		element.html( str_replace(element.html(), outerHtml, innerSearchText) );
	});
};
/**
 * If full width of page for example is less then width of our book - le't stry to adjust book width
 * @param {int} boundWidth Width to adjust for, current window width for example
 * @param {bool} areaOnly Do adjust only for book area (true) or for book too (false)
 */
dpsBook.prototype.adjustWidth = function(boundWidth, areaOnly) {
	if(!parseInt(boundWidth)) return;
	var bookHtml = this.getHtml()
	,	bookStage = this.getStage()
	,	bookShell = this.getBookShell()
	,	toolbar = this.getToolbar()
	,	stageWidth = bookStage.width()
	,	stageHeight = bookStage.height()
	,	shellHeight = bookShell.height()
	,	stageSizeUpdated = false
	,	newShellHeight = (boundWidth / stageWidth) * shellHeight
	,	newStageHeight = (boundWidth / stageWidth) * stageHeight
	,	toolbarHeight = toolbar.height()
	,	kWidth = 0;	// k == koeficient
	if(boundWidth < stageWidth) {
		this._setOriginalSizes(stageWidth, stageHeight, shellHeight);
	}
	if(this._orStageSize.height) {
		kWidth = (boundWidth / this._orStageSize.width);
		newStageHeight = kWidth * this._orStageSize.height + (toolbarHeight / kWidth);
		newShellHeight = kWidth * this._orShellSize.height/* + (toolbarHeight / kWidth)*/;
	}
	if(boundWidth < stageWidth) {
		bookStage.width( boundWidth ).height( newStageHeight );
		bookShell.height( newShellHeight );
		stageSizeUpdated = true;
		this._stageSizeChanged = true;
	} else if(this._orStageSize.width && this._stageSizeChanged) {
		if(this._orStageSize.width > boundWidth) {
			bookStage.width( boundWidth ).height( newStageHeight );
			bookShell.height( newShellHeight );
		} else {
			bookStage.width( this._orStageSize.width ).height( this._orStageSize.height );
			bookShell.height( this._orShellSize.height );
			this._orStageSize.width = 0;
			this._orStageSize.height = 0;
		}
		//this._stageSizeChanged = false;
		stageSizeUpdated = true;
	}
	//alert(stageWidth+ ','+ boundWidth);

	//if(stageWidthUpdated) {
		var bookWidth = bookHtml.width()
		,	bookSizeUpdated = false
		,	adjustByWidth = boundWidth;
		if(adjustByWidth > stageWidth) {
			adjustByWidth = stageWidth;
		}
		if(areaOnly) {
			bookSizeUpdated = true;
		} else {
			if(adjustByWidth < bookWidth) {
				if(!this._orBookSize.width) {
					this._orBookSize.width = bookWidth;
				}
				if(!this._orBookSize.height) {
					this._orBookSize.height = bookHtml.height();
				}
				var newBookHeight = (adjustByWidth / this._orBookSize.width) * this._orBookSize.height;
				bookHtml.turn('size', adjustByWidth, newBookHeight);
				bookSizeUpdated = true;
			} else if(this._orBookSize.width) {
				if(this._orBookSize.width > adjustByWidth) {
					var newBookHeight = (adjustByWidth / this._orBookSize.width) * this._orBookSize.height;
					bookHtml.turn('size', adjustByWidth, newBookHeight);
				} else {
					bookHtml.turn('size', this._orBookSize.width, this._orBookSize.height);
					this._orBookSize.width = 0;
				}
				bookSizeUpdated = true;
			}
		}
		if(bookSizeUpdated) {
			this.center();
			if(!stageSizeUpdated) {
				this._setOriginalSizes(stageWidth, stageHeight, shellHeight);
				kWidth = (adjustByWidth / this._orBookSize.width);
				newStageHeight = kWidth * this._orStageSize.height + (toolbarHeight / kWidth);
				newShellHeight = kWidth * this._orShellSize.height/* + (toolbarHeight / kWidth)*/;
				bookStage.height( newStageHeight );
				bookShell.height( newShellHeight );
			}
			this._toolbarSwitchResponsive(toolbar, adjustByWidth);
		}
		if(areaOnly)
			this.setBookTop(bookHtml);
	//}
};
dpsBook.prototype._toolbarSwitchResponsive = function(toolbar, adjustByWidth, reinit) {
	reinit = reinit ? reinit : false;
	if(this._bookData.toolbar && this._bookData.toolbar.switchResponsive) {
		var manipulateBtns = ['.dpsTbCenter .dpsTbBtnFirst', '.dpsTbCenter .dpsTbBtnLast', '.dpsTdPageNumForm'];
		if((adjustByWidth <= this._bookData.toolbar.switchResponsive && !this._toolbarInResponsive) || reinit) {
			toolbar.find( manipulateBtns.join(',') ).hide();
			toolbar.find('.dpsTbCenter .dpsTbBtnPrev, .dpsTbCenter .dpsTbBtnNext').each(function() {
				jQuery(this).parents('.btnContainer:first').css('vertical-align', 'top');
			});
			toolbar.addClass('dpsTbResponsiveOn');
			this._toolbarInResponsive = true;
		} else if(adjustByWidth > this._bookData.toolbar.switchResponsive && this._toolbarInResponsive) {
			toolbar.find( manipulateBtns.join(',') ).show();
			toolbar.find('.dpsTbCenter .dpsTbBtnPrev, .dpsTbCenter .dpsTbBtnNext').each(function() {
				jQuery(this).parents('.btnContainer:first').css('vertical-align', 'baseline');
			});
			toolbar.removeClass('dpsTbResponsiveOn');
			this._toolbarInResponsive = false;
		}
	}
};
// Reactivate toolbar responsive mode after book reinit
dpsBook.prototype._toolbarReinitResponsive = function() {
	if(this._toolbarInResponsive) {
		this._toolbarInResponsive = false;
		this._toolbarSwitchResponsive( this.getToolbar(), jQuery(window).width(), true );
	}
};
dpsBook.prototype._setOriginalSizes = function(stageWidth, stageHeight, shellHeight) {
	if(!this._orStageSize.width)
		this._orStageSize.width = stageWidth;
	if(!this._orStageSize.height)
		this._orStageSize.height = stageHeight;
	if(!this._orShellSize.height)
		this._orShellSize.height = shellHeight;
};
dpsBook.prototype._adjustPosForZommed = function() {
	var bookHtml = this.getHtml()
	,	pagsNum = this._getDisplayPagesNum()
	,	bookHeight = this.get('prevHeight') * this._zoomedLevel
	,	top = bookHeight / 2.3 + 'px'
	,	side = pagsNum && pagsNum.length == 1 && pagsNum[0] == 1 ? 'right' : 'left';

	bookHtml.css({
		'top': top
	,	'left': '0'
	,	'right': 'auto'
	,	'margin-right': 'auto'
	,	'margin-left': 'auto'
	,	'margin-bottom': '20px'
	,	'transform-origin': side + ' center 0px'
	});

	if(pagsNum && pagsNum.length == 1 && pagsNum[0] != 1) {
		// fix of centering last page after turning
		setTimeout(function() {
			bookHtml.css({ 'margin-left': 'auto' });
		}, 50);
	}
};
dpsBook.prototype.turnFirst = function() {
	this.getHtml().turn('page', 1);
	this._checkMaximizeBookRestore();
};
dpsBook.prototype.turnLast = function() {
	var bookData = this.getData();

	this._checkMaximizeBookRestore();
	this.getHtml().turn('page', bookData.pages.length);
};
dpsBook.prototype.turnNext = function() {
	var bookHtml = this.getHtml()
	,	bookData = this.getData()
	,	currentPage = bookHtml.turn('page');

	if(currentPage < bookData.pages.length) {
		bookHtml.turn('next');
		if(this._zoomEnabled || this._fullscreenEnabled) {
			this._imgSetZoomSizes();
		}
	}
	this._checkMaximizeBookRestore();
};
dpsBook.prototype.turnPrev = function() {
	var bookHtml = this.getHtml()
	,	currentPage = bookHtml.turn('page');

	if(currentPage > 1) {
		bookHtml.turn('previous');
		if(this._zoomEnabled || this._fullscreenEnabled) {
			this._imgSetZoomSizes();
		}
	}
	this._checkMaximizeBookRestore();
};
dpsBook.prototype.getSocLinks = function() {
	return this.getToolbar().find('.dpsTbSlShell');
};
dpsBook.prototype.toggleSocLinks = function(newValue) {
	if(typeof(newValue) !== 'undefined') {
		newValue
			? this.getSocLinks().show( this._anim )
			: this.getSocLinks().hide( this._anim );
	} else
		this.getSocLinks().toggle( this._anim );
};
dpsBook.prototype._removeFakePages = function() {
	if(!this._bookData.params['en_fake_pages']) return;
	var bookShell = this.getBookShell();
	bookShell.find('.dpsFakePage').remove();
};
dpsBook.prototype._createFakePages = function() {
	if(!this._bookData.params['en_fake_pages']) return;
	var bookHtml = this.getHtml();
	if(!bookHtml) return;	// This mean book was not created
	var bookShell = this.getBookShell()
	,	currentPage = bookHtml.turn('page')
	,	fakeI = currentPage - 1
	,	bookWidth = bookHtml.width()
	,	bookHeight = bookHtml.height()
	,	shellWidth = bookShell.width()
	,	shellHeight = bookShell.height()
	,	fakeTop = (shellHeight - bookHeight) / 2
	,	fakeLeft = ((shellWidth - bookWidth) / 2) + bookWidth - 1	// 1 - for border
	,	i = 0
	,	fakeWidth = 4
	,	pageNums = this._getDisplayPagesNum()
	,	pagesPerFake = 10
	,	pagesCount = this._pagesCount;
	this._removeFakePages();
	if(pageNums.length == 1) {
		if(pageNums[0] != 1)	// If we see 1 page - then this is begining or end, if 1 - begining, else - end, and we don't need fake pages here
			return;
		fakeLeft -= bookWidth/4;
	}

	//this._pagesCount = 900;
	bookHeight -= 3;
	fakeLeft = Math.floor(fakeLeft);
	fakeTop = Math.floor(fakeTop);
	bookHeight = Math.floor(bookHeight);
	do {
		var clickTurnTo = fakeI + 1 + pagesPerFake
		,	newFakePage = jQuery('<div class="dpsFakePage" data-page="'+ clickTurnTo+ '"/>')
		,	newFakePageCss = {
				'top': fakeTop
			,	'height': bookHeight+ 'px'
		};
		if(this._bookData.params['transition_direction'] == 'rtl')
			newFakePageCss.right = fakeLeft;
		else
			newFakePageCss.left = fakeLeft

		newFakePage.css( newFakePageCss ).addClass(this._bookData.params['transition_direction']);
		newFakePage.click(function(){
			var turnToPage = parseInt(jQuery(this).data('page'));
			if(turnToPage && bookHtml) {
				if(turnToPage > pagesCount)
					turnToPage = pagesCount;

				setTimeout(function(){
					bookHtml.turn('page', turnToPage);
				}, 100);

			}

		});
		bookShell.append(newFakePage);
		fakeLeft += fakeWidth;
		fakeTop += 2;
		bookHeight -= 4;
		fakeI += pagesPerFake;	// one fake peg per each 10 original pages
		i++;
	} while(fakeI < pagesCount);
};
dpsBook.prototype.getExportLinks = function() {
	return this.getToolbar().find('.dpsTbExportShell a');
};
dpsBook.prototype.toggleExportLinks = function(newValue) {
	if(typeof(newValue) !== 'undefined') {
		newValue
			? this.getExportLinks().show( this._anim )
			: this.getExportLinks().hide( this._anim );
	} else
		this.getExportLinks().toggle( this._anim );
};
dpsBook.prototype._checkBorderShadow = function() {
	if(!this._bookData.params['en_page_shadow']) return;
	var pageNums = this._getDisplayPagesNum()
	,	bookHtml = this.getHtml();
	if(!bookHtml) return;	// This mean book was not created
	if(pageNums.length == 1) {
		bookHtml.find('.page-wrapper:visible').addClass('borderShadowed');
	} else {
		bookHtml.addClass('borderShadowed');
	}
};
dpsBook.prototype._removeBorderShadow = function() {
	if(!this._bookData.params['en_page_shadow']) return;
	var bookHtml = this.getHtml();
	bookHtml
		.removeClass('borderShadowed')
		.find('.page-wrapper')
		.removeClass('borderShadowed');
};
