<?php
class pageDps extends moduleDps {
	public function init() {
		parent::init();
		// Add books filter for pages
		add_action('restrict_manage_posts', array($this, 'addPagesSort'));
		// Product meta box
		add_action('add_meta_boxes', array($this, 'addPageMetaBoxes'));
		// Open our menu for Books menu item
		add_filter('parent_file', array($this, 'checkPagesAdminParentPage'));
		// Add custom posts list columns in admin area
		add_filter('manage_edit-'. DPS_PAGE. '_columns', array($this, 'initPageAdminColumns'));
		add_action('manage_posts_custom_column' , array($this, 'addPageAdminColumns') );
		// Add custom posts actions for admin list
		add_filter('post_row_actions', array($this, 'rebuildAdminTblActions'), 10, 2);

		add_action('add_meta_boxes', array($this, 'addEditPageScripts'));
		add_filter('redirect_post_location', array($this, 'addQueryToOpenPreview'), 10, 2);
		add_filter('post_updated_messages', array($this, 'addPageUpdateMessages'));
		// Save custom page add manually usage statistics
		add_action('wp_insert_post_data', array($this, 'savePageStatistic'), 99, 2);
	}
	public function loadAdminAssets() {
		add_action('init', array($this, 'loadCurrentSearchBookId'));
		frameDps::_()->addScript('admin.page', $this->getModPath(). 'js/admin.page.js');
	}
	public function loadCurrentSearchBookId() {
		$searchBookId = reqDps::getVar(DPS_BOOK);
		if(!empty($searchBookId)) {
			$isSlug = !is_numeric($searchBookId);
			if(!$isSlug) {
				// This can be numeric slug - check it
				$book = frameDps::_()->getModule('book')->getModel()->getById($searchBookId, false, true);
				if(!$book)
					$isSlug = true;
			}
			if($isSlug) {	// Search is by alias
				$book = frameDps::_()->getModule('book')->getModel()->getById($searchBookId, false, true, 'slug');
				if($book) {
					$searchBookId = $book->term_id;
				}
			}
			if($searchBookId) {
				frameDps::_()->addJSVar('admin.page', 'dpsSearchBookId', $searchBookId);
			}
		}

	}
	public function addPagesSort() {
		global $typenow;
        if($typenow == DPS_PAGE){
			$tax = get_taxonomy(DPS_BOOK);
			$booksOpts = array('' => sprintf(__('Show Pages for All %s', DPS_LANG_CODE), $tax->labels->name));
			$booksList = frameDps::_()->getModule('book')->getModel()->getList(array(), false);
			if(!empty($booksList)) {
				foreach($booksList as $book) {
					$booksOpts[ $book->slug ] = $book->name. ' ('. $book->count. ')';
				}
			}
			echo htmlDps::selectbox(DPS_BOOK, array('options' => $booksOpts, 'value' => reqDps::getVar(DPS_BOOK)));
        }
	}
	public function addPageMetaBoxes() {
		add_meta_box('dpsBatchAddButtons', __('Batch add pages'), array($this, 'displayBatchAddPagesButtons'), DPS_PAGE, 'side', 'high', DPS_LANG_CODE);
	}
	public function displayBatchAddPagesButtons() {
		global $post;
		$this->getView()->displayBatchAddPagesButtons($post->ID);
	}
	public function checkPagesAdminParentPage($parent_file) {
		if(frameDps::_()->getModule('pages')->isAdminPagesPage()) {
			$parent_file = frameDps::_()->getModule('adminmenu')->getMainSlug();
		}
		return $parent_file;
	}
	public function initPageAdminColumns($cols) {
		$keys = array('cb', 'title', 'preview', 'books', 'date');
		$cols['books'] = __('Used in Publications', DPS_LANG_CODE);
		$cols['preview'] = __('Preview', DPS_LANG_CODE);
		unset($cols['tags']);
		$sortedCols = array();
		foreach($keys as $key) {
			$sortedCols[ $key ] = $cols[ $key ];
		}
		return $sortedCols;
	}
	public function addPageAdminColumns($column) {
		global $post;
		if($post->post_type == DPS_PAGE) {
			$out = '';
			switch($column) {
				case 'books':
					$books = $this->getModel()->getPageBooks($post->ID);
					if(empty($books) || is_wp_error($books)) {
						$out = '<span class="description">'. __('This page is not in any book', DPS_LANG_CODE). '</span>
							<br />
							<a target="_blank" href="'. frameDps::_()->getModule('book')->getLinkBookAdd(). '">'. __('Create Publication', DPS_LANG_CODE). '</a>';
					} else {
						$outArr = array();
						foreach($books as $book) {
								$outArr[] = '<a target="_blank" href="'. frameDps::_()->getModule('book')->getLinkBookEdit($book->term_id). '">'
										. $book->name
										.'</a>';

						}
						$out = implode('<br />', $outArr);
					}
					break;
				case 'preview':
					echo '<div>'. $post->post_content .'</div>';
					break;
			}
			if(!empty($out))
				echo $out;
		}
	}
	public function rebuildAdminTblActions($actions, $post) {
		if($post->post_type == DPS_PAGE) {
			$books = $this->getModel()->getPageBooks($post->ID);
			if(!empty($books) && !is_wp_error($books)) {
				$book = array_shift($books);
				$actions['page_preview'] = '<a target="_blank" title="'. __('Preview Page', DPS_LANG_CODE). '" href="'. uriDps::mod('book', 'preview', array('id' => $book->term_id)). '#page'. $post->ID. '">'. __('Preview', DPS_LANG_CODE). '</a>';
			}
		}
		return $actions;
	}
	public function addQueryToOpenPreview($location, $post_id) {
		$openPagePreview = (int) reqDps::getVar('open_page_preview');
		if($openPagePreview && get_post_type($post_id) == DPS_PAGE) {
			$bookPreviewId = 0;
			$books = $this->getModel()->getPageBooks($post_id);
			if(!empty($books)) {
				$book = array_shift($books);
				if($book && is_object($book)) {
					$bookPreviewId = $book->term_id;
				}
			}
			if($bookPreviewId) {
				$location .= '&open_book_page_preview_id='. $bookPreviewId;
			} else {	// No books for page were found
				$location = add_query_arg('message', 501, $location);
			}
		}
		return $location;
	}
	public function addEditPageScripts($post) {
		if(get_post_type($post) == DPS_PAGE) {
			$openBookPagePreviewId = (int) reqDps::getVar('open_book_page_preview_id');
			frameDps::_()->addScript('admin.page.edit', $this->getModPath(). 'js/admin.page.edit.js');
			frameDps::_()->addJSVar('admin.page.edit', 'dpsPage', (array)$post);
			frameDps::_()->addJSVar('admin.page.edit', 'dpsBookSelectedId', (string) (int) reqDps::getVar(DPS_BOOK));
			frameDps::_()->addJSVar('admin.page.edit', 'dpsOpenPagePreview', (string) $openBookPagePreviewId);
			if($openBookPagePreviewId) {
				frameDps::_()->addJSVar('admin.page.edit', 'dpsPagePreviewUrl', frameDps::_()->getModule('book')->getLinkPreviewBook(array('id' => $openBookPagePreviewId, 'page' => $post->ID)));
			}
			frameDps::_()->addScript('admin.page', $this->getModPath(). 'js/admin.page.js');
		}
	}
	public function addPageUpdateMessages($messages) {
		$messages[ DPS_PAGE ] = array();
		$messages[ DPS_PAGE ][ 501 ] = __('Select publication before preview page', DPS_LANG_CODE);
		foreach($messages[ DPS_PAGE ] as $i => $msg) {
			if($i >= 500) {	// Error
				$messages[ DPS_PAGE ][$i] = '<div class="dpsErrorMsg">'. $messages[ DPS_PAGE ][$i]. '</div>';
			}
		}
		return $messages;
	}
	public function getLinkPageAdd() {
		return admin_url('post-new.php?post_type='. DPS_PAGE);
	}
	public function savePageStatistic($data, $postarr) {
		if(defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) return $data;
		if($data['post_type'] != DPS_PAGE) return $data;
		// Only if user save post, not trigger for other save post actions
		$scriptFile = basename($_SERVER['PHP_SELF']);
		if($scriptFile !== 'post.php') return $data;
		return $data;
	}
}
