<?php
class pageModelDps extends modelDps {
	private function _isQueryBookPages($wpQuery) {
		if(isset($wpQuery->query_vars['post_type']) 	// Additional check
			&& $wpQuery->query_vars['post_type'] == DPS_PAGE
		) {
			return true;
		}
		return false;
	}
	public function addJoinTables($join, $wpQuery) {
		if($this->_isQueryBookPages($wpQuery)) {
			$join .= ' '. dbDps::prepareQuery('LEFT JOIN `@__page_sort_order` ON `@__page_sort_order`.`page_id` = `#__posts`.`ID`');
			$join .= ' '. dbDps::prepareQuery('LEFT JOIN `@__page_as_img` ON `@__page_as_img`.`page_id` = `#__posts`.`ID`');
		}
		return $join;
	}
	public function addOrderCondition($orderby, $wpQuery) {
		if($this->_isQueryBookPages($wpQuery)) {
			$orderby = dbDps::prepareQuery('`@__page_sort_order`.`sort_order`, `#__posts`.`ID` ASC');
		}
		return $orderby;
	}
	public function addFieldsToQuery($fields, $wpQuery) {
		if($this->_isQueryBookPages($wpQuery)) {
			$fields .= ', '. dbDps::prepareQuery('`@__page_as_img`.`attach_id`');
		}
		return $fields;
	}
	public function getListForBookByBookId($id, $opts = array()) {
		if((int) $id) {
			$bookParams = array('book_id' => $id);
			$opts = $opts ? $opts : array();
			if(isset($opts['get_all_pages']) && $opts['get_all_pages']) {
				$bookParams['post_status'] = 'any';
			}
			$pages = $this->get($bookParams);
			if(!empty($pages)) {
				return $pages;
			} else
				$this->pushError(__('Can not find any pages for provided Publication ID', DPS_LANG_CODE));
		} else
			$this->pushError (__('Invalid Publication ID', DPS_LANG_CODE));
		return array();
	}
	public function getById($id, $params = array()) {
		if((int)$id) {
			$params = isset($params) && !empty($params) ? $params : array();
			$page = $this->get(array_merge($params, array('id' => $id)));
			if($page) {
				return $page;
			} else
				$this->pushError (__('Can not find page by ID', DPS_LANG_CODE));
		} else
			$this->pushError (__('Invalid Page ID', DPS_LANG_CODE));
		return false;
	}
	public function get($d = array()) {
		$getPostsArgs = array(
			'post_type' => DPS_PAGE,
			'suppress_filters' => false,
			'posts_per_page' => -1,
		);
		if(isset($d['post_status']))
			$getPostsArgs['post_status'] = $d['post_status'];
		if(isset($d['book_id'])) {
			$getPostsArgs['tax_query'] = array(
				array(
					'taxonomy' => DPS_BOOK,
					'field' => 'term_id',
					'terms' => (int)$d['book_id']
				)
			);
		}
		if(isset($d['id'])) {
			$getPostsArgs['include'] = is_array($d['id']) ? array_map('intval', $d['id']) : (int) $d['id'];
		}
		add_filter('posts_join_request', array($this, 'addJoinTables'), 10, 2);
		add_filter('posts_orderby_request', array($this, 'addOrderCondition'), 10, 2);
		add_filter('posts_fields_request', array($this, 'addFieldsToQuery'), 10, 2);

		$pages = get_posts( $getPostsArgs );

		remove_filter('posts_fields_request', array($this, 'addFieldsToQuery'));
		remove_filter('posts_orderby_request', array($this, 'addOrderCondition'));
		remove_filter('posts_join_request', array($this, 'addJoinTables'));

		if(!empty($pages)) {
			foreach($pages as $i => $page) {
				$pages[ $i ] = $this->_afterGet($pages[ $i ], $d);
			}
			// If get post by ID - then return only first value
			return isset($d['id']) && !is_array($d['id'])
				? array_shift($pages)
				: $pages;
		}
		return false;
	}
	public function remove($d = array()) {
		$ids = array();
		if(isset($d['id'])) {
			$ids = is_array($d['id']) ? $d['id'] : array((int) $d['id']);
		}
		if(!empty($ids)) {
			foreach($ids as $id) {
				$this->removeById($id);
			}
			frameDps::_()->getTable('page_as_img')->delete(array('additionalCondition' => 'page_id IN ('. implode(', ', $ids). ')'));
			frameDps::_()->getTable('page_sort_order')->delete(array('additionalCondition' => 'page_id IN ('. implode(', ', $ids). ')'));
			return true;
		} else
			$this->pushError (__('Empty or invalid ID', DPS_LANG_CODE));
		return false;
	}
	public function removeById($id) {
		return wp_trash_post($id);
	}
	private function _afterGet($page, $params = array()) {
		if(isset($params['includeEditLink'])) {
			$page->edit_link = frameDps::_()->getModule('book')->getLinkPageEdit($page->ID);
		}
		return $page;
	}
	public function getPageBooks($pageId) {
		return get_the_terms($pageId, DPS_BOOK);
	}
	public function getTotalCount() {
		return wp_count_posts(DPS_PAGE)->publish;
	}
	private function _getNewImgPath($path, $pref, $num) {
		return str_replace(".$pref", "-$num.$pref", $path);
	}
	private function _createImgRes($imgPath, $imgType) {
		$readFuncName = 'imagecreatefrom'. $imgType;
		if(function_exists($readFuncName)) {
			$source = $readFuncName($imgPath);
		} else {
			$imgContent = @file_get_contents($imgPath);
			$source = imagecreatefromstring($imgContent);
		}
		return $source;
	}
	private function _saveImg($imgSrc, $imgPath, $imgType, $imgPref, $num) {
		$createFuncName = 'image'. $imgType;
		if(function_exists($createFuncName)) {
			$newImgPath = $this->_getNewImgPath($imgPath, $imgPref, $num);
			$createFuncName($imgSrc, $newImgPath);
			return $newImgPath;
		}
		return false;
	}
	public function split($d = array()) {
		$id = isset($d['id']) ? (int) $d['id'] : 0;
		if($id) {
			$page = $this->getById($id);
			if($page) {
				$imgIds = utilsDps::parseImgIds($page->post_content);
				if(!empty($imgIds)) {
					$imgId = array_shift($imgIds);
					$fullImgData = wp_get_attachment_image_src($imgId, 'full');
					$fullImgData[0] = utilsDps::checkAttachUrl($fullImgData[0]);
					if(!empty($fullImgData[0])) {
						$imgPath = utilsDps::getUploadFilePathFromUrl($fullImgData[0]);
						$imgMime = strtolower(get_post_mime_type($imgId));
						if(strpos($imgMime, 'image/') === 0) {
							$imgType = str_replace('image/', '', $imgMime);
							$imgPref = $imgType == 'jpeg' ? 'jpg' : $imgType;
							$source = $this->_createImgRes($imgPath, $imgType);
							if($source !== false) {
								$addedIds = array();
								$widthHeight = getimagesize($imgPath);
								$halfOfImg = ceil($widthHeight[0] / 2);
								$newImg = imagecreatetruecolor($halfOfImg, $widthHeight[1]);
								$secondHalf = false;
								$pageBooks = $this->getPageBooks($id);
								$pageBook = array_shift($pageBooks);
								$pageBookId = $pageBook->term_id;
								for($x = 0; $x < $widthHeight[0]; $x++) {
									if($x == $halfOfImg) {
										$newImgPath = $this->_saveImg($newImg, $imgPath, $imgType, $imgPref, '1');
										imagedestroy($newImg);
										$addedIds[] = $this->addPageFromImg(array(
											'img_url' => utilsDps::getUploadUrlFromFilePath($newImgPath),
											'title' => $page->post_title. '-1',
											'book_id' => $pageBookId,
										));
										$newImg = imagecreatetruecolor($halfOfImg - 1, $widthHeight[1]);
										$secondHalf = true;
									}
									for($y = 0; $y < $widthHeight[1]; $y++) {
										$rgb = imagecolorat($source, $x, $y);
										$pix = imagecolorsforindex($source, $rgb);
										$rgb = imageColorAllocate($newImg, $pix['red'], $pix['green'], $pix['blue']);
										$newX = $secondHalf ? ($x - $halfOfImg) : $x;
										imagesetpixel($newImg, $newX, $y, $rgb);
									}
								}
								$newImgPath = $this->_saveImg($newImg, $imgPath, $imgType, $imgPref, '2');
								imagedestroy($newImg);
								$addedIds[] = $this->addPageFromImg(array(
									'img_url' => utilsDps::getUploadUrlFromFilePath($newImgPath),
									'title' => $page->post_title. '-2',
									'book_id' => $pageBookId,
								));
								// Update sort order to insert ordering of new pages into old page place
								if(isset($d['sort_order']) && !empty($d['sort_order'])) {
									frameDps::_()->getModule('book')->getModel()->updatePagesSortOrder($pageBookId, $d['sort_order']);
								}
								$pagesSortOrder = frameDps::_()->getModule('book')->getModel()->getPagesSortOrderByPageId($pageBookId);
								$newPagesSortOrder = array();
								$incSortOrder = 0;
								foreach($pagesSortOrder as $pageId => $sortOrder) {
									if($pageId == $id) {	// Current page - it will be replaced by 2 new pages
										$incSortOrder = 1;
										$newPagesSortOrder[ $addedIds[0] ] = $sortOrder;	//[1] then [0] - because last added pages should be displayed at first
										$newPagesSortOrder[ $addedIds[1] ] = $sortOrder + $incSortOrder;
									} else
										$newPagesSortOrder[ $pageId ] = $sortOrder + $incSortOrder;
								}
								frameDps::_()->getModule('book')->getModel()->updateSortOrderFromArray($pageBookId, $newPagesSortOrder);
								$this->remove(array('id' => $id));
								return $addedIds;
							} else
								$this->pushError (__('Can\'t create image gd recourse', DPS_LANG_CODE));
						} else
							$this->pushError (__('Is selected - image type?', DPS_LANG_CODE));
					} else
						$this->pushError (__('Can\'t find src for image', DPS_LANG_CODE));
				} else
					$this->pushError (__('There are no images in selected page', DPS_LANG_CODE));
			} else
				$this->pushError (__('Can not find page', DPS_LANG_CODE));
		} else
			$this->pushError(__('Invalid ID', DPS_LANG_CODE));
		return false;
	}
	public function merge($d = array()) {
		$ids = isset($d['ids']) ? array_map('intval', $d['ids']) : 0;
		if($ids) {
			$pages = $this->get(array('id' => $ids));
			if(!empty($pages)) {
				$imgs = array();
				$firstPage = null;
				foreach($pages as $i => $p) {
					$pageImgIds = utilsDps::parseImgIds($p->post_content);
					if(!empty($pageImgIds)) {
						$imgId = array_shift($pageImgIds);
						$fullImgData = wp_get_attachment_image_src($imgId, 'full');
						$fullImgData[0] = utilsDps::checkAttachUrl($fullImgData[0]);
						if(!empty($fullImgData[0])) {
							$imgData = array(
								'imgPath' => utilsDps::getUploadFilePathFromUrl($fullImgData[0]),
								'imgMime' => strtolower(get_post_mime_type($imgId)),
							);
							$imgData['widthHeight'] = getimagesize($imgData['imgPath']);
							$imgData['imgType'] = str_replace('image/', '', $imgData['imgMime']);
							$imgData['imgPref'] = $imgData['imgType'] == 'jpeg' ? 'jpg' : $imgData['imgType'];
							$imgs[] = $imgData;
							if($i == 0) {	// Book for each pages - should be the same, so we can take it here, for first page only
								$firstPage = $p;

							}
						}
					}
				}
				if(count($imgs) > 1) {
					$pageBooks = $this->getPageBooks($firstPage->ID);
					$pageBook = array_shift($pageBooks);
					$pageBookId = $pageBook->term_id;
					$newWidth = $newHeight = 0;
					foreach($imgs as $i => $img) {
						$imgs[ $i ]['source'] = $this->_createImgRes($img['imgPath'], $img['imgType']);
						$newWidth += $img['widthHeight'][0];
						// Height of new image - will be minimum from selected images height as we don't want to leave empty space bellow image
						if(!$newHeight || $img['widthHeight'][1] < $newHeight)
							$newHeight = $img['widthHeight'][1];
					}
					$newImg = imagecreatetruecolor($newWidth, $newHeight);
					$startX = 0;
					foreach($imgs as $i => $img) {
						for($x = 0; $x < $img['widthHeight'][0]; $x++) {
							for($y = 0; $y < $newHeight; $y++) {
								$rgb = imagecolorat($img['source'], $x, $y);
								$pix = imagecolorsforindex($img['source'], $rgb);
								$rgb = imageColorAllocate($newImg, $pix['red'], $pix['green'], $pix['blue']);
								$newX = $x + $startX;
								imageSetPixel($newImg, $newX, $y, $rgb);
							}
						}
						imagedestroy($img['source']);
						$startX += $img['widthHeight'][0];
					}
					// Type, path name and other - can be taken from first image
					$firsImg = $imgs[0];
					$newImgPath = $this->_saveImg($newImg, $firsImg['imgPath'], $firsImg['imgType'], $firsImg['imgPref'], 'merged');
					imagedestroy($newImg);
					$addedId = $this->addPageFromImg(array(
						'img_url' => utilsDps::getUploadUrlFromFilePath($newImgPath),
						'title' => $firstPage->post_title. '-merged',
						'book_id' => $pageBookId,
					));
					if($addedId) {
						// Update sort order to insert ordering of new pages into old page place
						if(isset($d['sort_order']) && !empty($d['sort_order'])) {
							frameDps::_()->getModule('book')->getModel()->updatePagesSortOrder($pageBookId, $d['sort_order']);
						}
						$pagesSortOrder = frameDps::_()->getModule('book')->getModel()->getPagesSortOrderByPageId($pageBookId);
						$newPagesSortOrder = array();
						$decSortOrder = 0;
						foreach($pagesSortOrder as $pageId => $sortOrder) {
							if(in_array($pageId, $ids)) {	// All current pages will be erased and to it's place will be puted new one
								if(empty($decSortOrder)) {
									$newPagesSortOrder[ $addedId ] = $sortOrder;
								}
								$decSortOrder++;
							} else
								$newPagesSortOrder[ $pageId ] = $sortOrder - $decSortOrder;
						}
						frameDps::_()->getModule('book')->getModel()->updateSortOrderFromArray($pageBookId, $newPagesSortOrder);
						$this->remove(array('id' => $ids));
						return $addedId;
					} else
						$this->pushError (__('Can\'t create result image', DPS_LANG_CODE));

				} else
					$this->pushError (__('Nothing to merge', DPS_LANG_CODE));
			} else
				$this->pushError (__('Can not find pages', DPS_LANG_CODE));
		} else
			$this->pushError(__('Invalid IDs', DPS_LANG_CODE));
		return false;
	}
	public function createAttachFromUrl($url) {
		// Check the type of tile. We'll use this as the 'post_mime_type'.
		//$uploads = wp_upload_dir();
		//$filePath = $uploads['path'] . '/' . basename( $url );
		$filePath = utilsDps::getUploadFilePathFromUrl($url);
		$fileType = wp_check_filetype( basename( $url ), null );

		// Prepare an array of post data for the attachment.
		$attachment = array(
			'guid'           => $url,
			'post_mime_type' => $fileType['type'],
			'post_title'     => preg_replace( '/\.[^.]+$/', '', basename( $url ) ),
			'post_content'   => '',
			'post_status'    => 'inherit'
		);
		$attachId = wp_insert_attachment($attachment, $filePath, 0);

		require_once(ABSPATH . 'wp-admin/includes/image.php');
		$metadata = wp_generate_attachment_metadata($attachId, $filePath);
		wp_update_attachment_metadata($attachId, $metadata);

		return $attachId;
	}
	private function _setPageAsImg($pageId, $attachId, $imgUrl) {

		global $wpdb;
		$tableName = $wpdb->prefix . "dps_page_as_img";
		$arr = array('page_id' => $pageId,
		'attach_id' => $attachId,
		'img_url' => $imgUrl);
		$res = $wpdb->insert($tableName, $arr);
		//
		// frameDps::_()->getTable('page_as_img')->insert(array(
		// 	'page_id' => $pageId,
		// 	'attach_id' => $attachId,
		// 	'img_url' => $imgUrl,
		// ));
	}
	public function addPageFromImg($d = array()) {
		$newPageId = 0;
		if(isset($d['attach_id']) && $d['attach_id']) {
			$attachId = $d['attach_id'];
		} elseif(isset($d['img_url']) && $d['img_url']) {
			$attachId = $this->createAttachFromUrl( $d['img_url'] );
		}
		$attach = wp_get_attachment_image_src($attachId, 'full');
		if($attachId && $attach && $attach[0]) {
			$attach[0] = utilsDps::checkAttachUrl($attach[0]);
			$content = '<img src="'. $attach[0]. '" width="100%" height="auto" class="size-full wp-image-'. $attachId. '" />';
			$newPageId = $this->addPage(array(
				'title' => $d['title'],
				'book_ids' => array($d['book_id']),
				'content' => $content,
			));
			if($newPageId && is_numeric($newPageId)) {
				$this->_setPageAsImg($newPageId, $attachId, $attach[0]);
			}
		}
		return $newPageId;
	}
	public function addPage($d = array()) {
		$newPageData = array(
			'post_title' => $d['title'],
			'post_content' => $d['content'],
			'post_type' => DPS_PAGE,
			'post_status' => 'publish'
		);
		if(isset($d['book_ids']) && !empty($d['book_ids'])) {
			$newPageData['tax_input'] = array(DPS_BOOK => $d['book_ids']);
		}
		return wp_insert_post($newPageData);
	}
}
